/************************************************************************/
/* filterset.h - Filter defines						*/
/************************************************************************/
/*
 *  (c)2021,2022 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _FILTERSET_H_
#define _FILTERSET_H_

#include <stdint.h>
#include <cutil/slist.h>
#include <cutil/avltree.h>

#include <metric.h>


typedef enum {
	F_TYPE_NONE			= 0,
	F_TYPE_SUBSTRING,
	F_TYPE_REGEX,
	F_TYPE_AVLTREE,
	F_TYPE_V4CIDR,
	_F_TYPE_COUNT,
} filter_type;


typedef enum {
	F_ACTION_NONE			= 0,
	F_ACTION_ACCEPT,
	F_ACTION_LOG,
	F_ACTION_REJECT,
	_F_ACTION_COUNT,
} filter_action;


typedef struct _filter_t filter_t;
struct _filter_t {
	filter_type		type;
	filter_action		action;
	char			*match;
	void			*arg;

	// number of hits(matches) for this filter
	metric_t		hits;

	// score adjustment for match (signed integer)
	int32_t			score;
};


typedef struct _filterset_t filterset_t;
struct _filterset_t {
	char		*name;
	filter_type	type;
	filter_action	action;

	void		*impl;

};


// TODO
//typedef struct _filterset_iter_t filterset_iter_t;
//struct _filterset_iter_t {
//	filterset_t			*fs;
//
//	void				*iter;
//
//	filterset_entry_enum_callback	cb;
//	void				*cb_arg;
//};


typedef int (*filterset_entry_enum_callback)(filter_t *entry, void *arg);


filterset_t *filterset_new(char *name,
		filter_type type,
		filter_action action);
int filterset_destroy(filterset_t *fs);
int filterset_add_entry(filterset_t *fs, char *match, int32_t score);
int filterset_remove_entry(filterset_t *fs, char *match);
filter_t *filterset_match(filterset_t *fs, char *match);
filter_action filterset_test(filterset_t *fs, char *match);
int filterset_enumerate_entries(filterset_t *fs,
		filterset_entry_enum_callback cb,
		void *cb_arg);

filter_type filterset_string_to_type(char *str);
filter_action filterset_string_to_action(char *str);
char *filterset_type_to_string(filter_type type);
char *filterset_action_to_string(filter_action action);

//filterset_iter_t *filterset_iterator_new(filterset_t *fs);
//int filterset_iterator_free(filterset_iter_t *fsi);
//int filterset_iterator_next(filterset_iter_t *fsi);

#endif


