/************************************************************************/
/* v4cidr.h - v4cidr defines						*/
/************************************************************************/
/*
 *  (c)2021,2022 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _V4CIDR_H_
#define _V4CIDR_H_

#include <stdint.h>
#include <cutil/slist.h>


#define V4CIDR_TABLE_WIDTH_DEFAULT	65535


typedef struct _v4cidr_entry_t v4cidr_entry_t;
struct _v4cidr_entry_t {
	uint32_t	net;	// host byte order
	uint32_t	mask;	// host byte order
	void		*data;	// user data pointer
};


typedef struct _v4cidr_hashtable_t v4cidr_hashtable_t;
struct _v4cidr_hashtable_t {
	int		width;
	SList		**table;
};


v4cidr_hashtable_t *v4cidr_hashtable_new(int width);
int v4cidr_hashtable_free(v4cidr_hashtable_t *table);

int v4cidr_hashtable_add_cidr(v4cidr_hashtable_t *table,
						char *cidr,
						void *data);
int v4cidr_hashtable_remove_cidr(v4cidr_hashtable_t *table,
						char *cidr);
const v4cidr_entry_t *v4cidr_hashtable_locate(v4cidr_hashtable_t *table,
						char *ip);
void *v4cidr_hashtable_locate_get_data(v4cidr_hashtable_t *table,
						char *ip);
int v4cidr_hashtable_exists(v4cidr_hashtable_t *table,
						char *ip);



#endif



